package de.uni_marburg.powersort.data;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;

public class IntegerSerializer {
    public static final Path DATA_DIR = Paths.get("powersort-competition");

    private final int id;

    public static ObjectSupplier fromEnum(CompetitionEnum dataEnum) {
        if (!dataEnum.isCompetitionInput()) {
            throw new IllegalArgumentException();
        }

        String prefixedId = dataEnum.toString();
        IntegerSerializer serializer = new IntegerSerializer(Integer.parseInt(prefixedId.substring(CompetitionEnum.COMPETITION_PREFIX.length())));
        return serializer.getObjectSupplier();
    }

    public IntegerSerializer(int id) {
        this.id = id;
    }

    public ObjectSupplier getObjectSupplier() {
        Integer[] list = read();
        return new IntegerSupplier(list) {
            @Override
            public Integer[] getCopy() {
                return super.getCopy();
            }
        };
    }

    public Integer[] read() {
        Path path = getPath();
        String content;
        try {
            content = Files.readString(path);
        } catch (IOException e) {
            System.err.println(this + ": Could not read file " + path.toAbsolutePath());
            System.exit(1);
            return null;
        }

        content = content.strip();
        if (content.charAt(0) != '[' || content.charAt(content.length() - 1) != ']') {
            System.err.println(this + ": Illegal data format: Expected list of integers surrounded by square brackets.");
            System.exit(1);
            return null;
        }
        content = content.substring(1, content.length() - 1);

        // Remove spaces
        content = content.replaceAll(" ", "");

        String[] elements = content.split(",");
        try {
            return Arrays.stream(elements).map(Integer::valueOf).toArray(Integer[]::new);
        } catch (NumberFormatException e) {
            System.err.println(this + ": Number format exception: " + e.getMessage());
            System.exit(1);
            return null;
        }
    }

    public void write(Integer[] list) throws IOException {
        Path path = getPath();
        if (Files.exists(path)) {
            throw new IOException(this + ": Target file does already exist");
        }

        String[] elements = Arrays.stream(list).map(Object::toString).toArray(String[]::new);
        String content = "[" + String.join(",", elements) + "]";
        Files.writeString(path, content);
    }

    private Path getPath() {
        return DATA_DIR.resolve(Integer.toString(id));
    }

    @Override
    public String toString() {
        return "IntegerSerializer{" + id + "}";
    }
}
